#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>

/*

This is the brand new and bug-free IPv4 dotted format parser, which is also fast! (It's written in C, duh!)

Usage: ./isValid
<ip_addr>: dotted octet format, number can be bin, oct and dec, mixed is also supported
	bin number: start with 0b, e.g. 0b1010
	oct number: start with 0, e.g. 0377
	dec number: just write it normally (but remove leading 0s!)

*/


void get_input(char* input)
{
	int i;
	printf("Enter IP address for validation:\n");
	if (read(STDIN_FILENO, input, 1000) < 0) {
		fprintf(stderr, "Couldn't read stdin.\n");
	}
	// remove trailing carriage
	for(i=0; i < strlen(input); i++)
	{
		if(input[i] == '\n')
			input[i] = '\0';
	}
}

int validate_bin(char* number, int len)
{
	int i;
	int in_decimal = 0;
	for(i = 0; i < len; i++)
	{
		if(number[i] == '0')
			in_decimal += 0;
		else if(number[i] == '1')
			in_decimal += 1* pow(2, len-i);
		else
			return -1;
	}

	if(in_decimal < 0 || in_decimal > 255)
		return -1;
	return 0;
}

int validate_dec(char* number, int len)
{
	int i;
	int in_decimal = 0;
	for(i = 0; i < len; i++)
	{
		if(number[i] == '0')
			in_decimal += 0;
		else if(number[i] == '1')
			in_decimal += 1* pow(10, len-i);
		else if(number[i] == '2')
			in_decimal += 2* pow(10, len-i);
		else if(number[i] == '3')
			in_decimal += 3* pow(10, len-i);
		else if(number[i] == '4')
			in_decimal += 4* pow(10, len-i);
		else if(number[i] == '5')
			in_decimal += 5* pow(10, len-i);
		else if(number[i] == '6')
			in_decimal += 6* pow(10, len-i);
		else if(number[i] == '7')
			in_decimal += 7* pow(10, len-i);
		else if(number[i] == '8')
			in_decimal += 8* pow(10, len-i);
		else if(number[i] == '9')
			in_decimal += 9* pow(10, len-i);
		else
			return -1;
	}

	if(in_decimal < 0 || in_decimal > 255)
		return -1;
	return 0;
}

int validate_oct(char* number, int len)
{
	int i;
	int in_decimal = 0;
	for(i = 0; i < len; i++)
	{
		if(number[i] == '0')
			in_decimal += 0;
		else if(number[i] == '1')
			in_decimal += 1* pow(8, len-i);
		else if(number[i] == '2')
			in_decimal += 2* pow(8, len-i);
		else if(number[i] == '3')
			in_decimal += 3* pow(8, len-i);
		else if(number[i] == '4')
			in_decimal += 4* pow(8, len-i);
		else if(number[i] == '5')
			in_decimal += 5* pow(8, len-i);
		else if(number[i] == '6')
			in_decimal += 6* pow(8, len-i);
		else if(number[i] == '7')
			in_decimal += 7* pow(8, len-i);
		else
			return -1;
	}

	if(in_decimal < 0 || in_decimal > 255)
		return -1;
	return 0;
}


int main(int argc, char *argv[])
{
	char input[100];
	char* token = NULL;
	int valid = 0;
	int dots = -1;
	int prevLen = 0;

	get_input(input);

	// split input at "."s and process them separately
	for(int i = 0; i < strlen(input)+1; i++)
	{
		if(input[i] == '.' || input[i] == '\0')
		{
			// end of number found, validate it!
			if(token[0] == '0')
			{
				if(token[1] == 'b')
				{
					// binary
					if ( validate_bin(&token[2], strlen(token)-2) == -1 )
						valid = -1;
				}
				else
				{
					// octal
					if ( validate_oct(&token[1], strlen(token)-1) == -1 )
						valid = -1;
				}
			}
			else
			{
				// decimal
				if ( validate_dec(token, strlen(token)) == -1 )
					valid = -1;
			}
			// new supposed-number validated
			dots++;
			token = NULL;
		}
		else 
		{
			// still part of the number
			if(token != NULL)
			{
				prevLen = strlen(token);
				token = (char*)realloc(token, strlen(token)+1);
				token[prevLen] = input[i];
			}
			else
			{
				token = (char*)malloc(1);
				token[0] = input[i];
			}
		}
	}

	// we should have parsed 4 tokens
	if(dots != 3)
		valid = -1;

	if(valid == 0)
		printf("%s is a valid IPv4 address\n", input);
	else	
		printf("%s is NOT a valid IPv4 address\n", input);

	// fail-safe deault: deny
	return -1;	
}
